###Sketch Config
#type = PluginCompound
#class_name = 'Angle'
#menu_text = 'Angle'
#load_immediately = 1
###End

import Sketch
from Sketch import Point, SingularMatrix, Rotation, RegisterCommands
from Sketch.Graphics import handle
from Sketch.UI.command import AddCmd

from lib.autoshapelib import AutoShape, GenericEditor
from math import pi, cos, sin


class AngleEditor(GenericEditor):
    commands = []

    def ChangeAngle(self, delta):
        p2 = self.p2
        beta = p2.polar()[1]+delta*pi/180.
        p2 = Point(cos(beta), sin(beta))*abs(p2)
        return self.object.SetParameters({'p2':p2})
    
    def IncreaseN(self):
        return self.ChangeAngle(+1)
    AddCmd(commands, IncreaseN, '', key_stroke=('+'))

    def DecreaseN(self):
        return self.ChangeAngle(-1)
    AddCmd(commands, DecreaseN, '', key_stroke=('-'))

RegisterCommands(AngleEditor)


class Angle(AutoShape):
    class_name = 'Angle'
    p1 = Point(10,0)
    p2 = Point(0,10)

    def GetSnapPoints(self):
        return (Point(0,0),)
    
    def GetHandles(self):
        handles = []
        p1, p2 = self.p1, self.p2
        for p in (p1, p2):
            handles.append(handle.MakeNodeHandle(self.trafo(p)))

        for p in (0.5*p1, 0.5*p2):
            handles.append(handle.MakeControlHandle(self.trafo(p)))

        return handles
                
    def recompute(self):
        new = Sketch.CreatePath()
        newpaths = [new]
        p1, p2 = self.p1, self.p2

        new.AppendLine(p1)
        new.AppendLine((0,0))
        new.AppendLine(p2)
        
        new.Transform(self.trafo)
        alpha = p1.polar()[1]
        beta = p2.polar()[1]
        if self.objects:
            self.objects[0].SetPaths(newpaths)
            self.objects[1].SetAngles(alpha, beta)
        else:
            lines = Sketch.PolyBezier(tuple(newpaths))
            circle = Sketch.Ellipse(start_angle=alpha, end_angle=beta)
            self.set_objects([lines, circle])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))
            delta = pb-pa
                         
            p1, p2 = self.p1, self.p2
            if selection == 1:
                p1 = p1 + delta
            elif selection == 2:
                p2 = p2 + delta
            elif selection == 3:
                alpha = (0.5*p1+delta).polar()[1]
                p1 = Point(cos(alpha), sin(alpha))*abs(p1)
            elif selection == 4:
                alpha = (0.5*p2+delta).polar()[1]
                p2 = Point(cos(alpha), sin(alpha))*abs(p2)
                
            self.p1, self.p2 = p1, p2 
            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        self.p1, self.p2 = map(Point, data)

    def GetData(self):
        return map(list, (self.p1, self.p2))

    def Info(self):
        p1, p2 = self.p1, self.p2
        alpha = p1.polar()[1]
        p = Rotation(-alpha)(p2)
        angle = (p.polar()[1]/pi*180.)
        if angle < 0:
            angle = angle + 360
        angle = int(angle*1000.+0.5)/1000.
        return "Angle of %s degrees" % angle

    def Editor(self):
        return AngleEditor(self)        
