###Sketch Config
#type = PluginCompound
#class_name = 'Star'
#menu_text = 'Star'
#load_immediately = 1
###End

import Sketch
from Sketch import Point, SingularMatrix, RegisterCommands, NullUndo, Rotation
from Sketch.Graphics import handle
from Sketch.UI.command import AddCmd

from lib.autoshapelib import AutoShape, GenericEditor
from math import pi, sin, cos

class StarEditor(GenericEditor):
    commands = []

    def GetN(self):
        return self.object.n
    def SetN(self, n):
        return self.object.SetParameters({'n':n})
    
    def IncreaseN(self):
        n = self.GetN()
        return self.SetN(n+1)
    AddCmd(commands, IncreaseN, '', key_stroke=('+'))

    def DecreaseN(self):
        n = self.GetN()
        if n >= 4:
            return self.SetN(n-1)
        else:
            return NullUndo
    AddCmd(commands, DecreaseN, '', key_stroke=('-')) 

RegisterCommands(StarEditor)


class Star(AutoShape):
    class_name = 'Star'

    r1 = 10
    r2 = 20
    n = 6
    
    def GetSnapPoints(self):
        return self.trafo((0,0))
    
    def GetHandles(self):
        r1, r2, n = self.r1, self.r2, self.n

        handles = []

        alpha = 2*pi / n
        for i in range(n):
            beta = i * alpha 
            p = r1*cos(beta), r1*sin(beta)
            handles.append(handle.MakeNodeHandle(self.trafo(p)))

        for i in range(n):
            beta = (i+0.5) * alpha 
            p = r2*cos(beta), r2*sin(beta)            
            handles.append(handle.MakeControlHandle(self.trafo(p)))

        return handles
                
    def recompute(self):
        new = Sketch.CreatePath()
        newpaths = [new]

        r1, r2, n = self.r1, self.r2, self.n
        
        alpha = 2*pi / n
        for i in range(n):
            beta = i * alpha 
            p = r1*cos(beta), r1*sin(beta)
            new.AppendLine(p)
            beta = (i+0.5) * alpha
            p = r2*cos(beta), r2*sin(beta)
            new.AppendLine(p)

        new.AppendLine((r1,0))
        new.Transform(self.trafo)    
        if self.objects:
            self.objects[0].SetPaths(newpaths)
        else:
            obj = Sketch.PolyBezier(tuple(newpaths))
            self.set_objects([obj])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))

            r1, r2, n = self.r1, self.r2, self.n
            
            alpha = 2*pi / n

            ra, beta = pa.polar()
            rot = Rotation(-beta)
            rb, gamma = rot(pb).polar()
    
            if selection <= n:
                self.r1 = min(r2, r1+rb-ra)
            else:
                n = n+int(2* gamma / alpha)    
                if n >= 3:
                    self.n = n  

            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        self.r1, self.r2, self.n = data

    def GetData(self):
        return (self.r1, self.r2, self.n)

    def Info(self):
        return _("Star with %s arms") % self.n

    def Editor(self):
        return StarEditor(self)
